from typing import List
from pydantic import BaseModel, Field


class AgentConfig(BaseModel):
	role: str = Field(..., description="The role of the agent")
	goal: str = Field(..., description="The goal of the agent")
	backstory: str = Field(..., description="The backstory of the agent")
	tools: List[str] = Field(..., description="The tools used by the agent")

class TaskConfig(BaseModel):
	description: str = Field(..., description="The description of the task")
	expected_output: str = Field(..., description="The expected output of the task")
	agent: AgentConfig = Field(..., description="The agent responsible for the task")

class CrewConfig(BaseModel):
	tasks: List[TaskConfig] = Field(..., description="The tasks to be performed by the crew")


from crewai.agent import Agent
from crewai.tools.agent_tools import AgentTools
from crewai.utilities import I18N

class HierarchicalManagerAgent:
	def __init__(self, llm, agents, verbose):
		i18n = I18N()
		self.agent = Agent(
									role=i18n.retrieve("hierarchical_manager_agent", "role"),
									goal=i18n.retrieve("hierarchical_manager_agent", "goal"),
									backstory=i18n.retrieve("hierarchical_manager_agent", "backstory"),
									tools=AgentTools(agents=agents).tools(),
									llm=llm,
									verbose=verbose,
								)


from crewai.agent import Agent
from crewai.utilities import i18n

class PlanningManagerAgent:
	def __init__(self, llm, verbose):
		self.agent = Agent(
									role=i18n.retrieve("planning_manager_agent", "role"),
									goal=i18n.retrieve("planning_manager_agent", "goal"),
									backstory=i18n.retrieve("planning_manager_agent", "backstory"),
									verbose=verbose,
									llm=llm,
								)


from .... import Agent, Crew, Process, Task
from ....project import CrewBase, agent, crew, task

from .crew_config import CrewConfig

@CrewBase
class PlanningCrewCrew():
  """PlanningCrew crew"""
  agents_config = '../internal/crew/planning_crew/config/agents.yaml'
  tasks_config = '../internal/crew/planning_crew/config/tasks.yaml'

  @agent
  def project_manager(self) -> Agent:
    return Agent(
      config=self.agents_config['project_manager'],
      allow_delegation=False,
      verbose=True
    )

  @agent
  def resource_manager(self) -> Agent:
    return Agent(
      config=self.agents_config['resource_manager'],
      allow_delegation=False,
      verbose=True
    )

  @task
  def task_decomposition(self) -> Task:
    return Task(
      config=self.tasks_config['task_decomposition'],
      agent=self.project_manager(),
      output_pydantic=CrewConfig
    )

  @task
  def resource_allocation(self) -> Task:
    return Task(
      config=self.tasks_config['resource_allocation'],
      agent=self.resource_manager(),
      output_pydantic=CrewConfig
    )

  @crew
  def crew(self) -> Crew:
    """Creates the PlanningCrew crew"""
    return Crew(
      agents=[self.project_manager(), self.resource_manager()],
      tasks=[self.task_decomposition(), self.resource_allocation()],
      process=Process.sequential
    )